#include "ConfigOpr.h"
#include <cassert>

constexpr const char* KEY_PWD = "Password";
constexpr const char* KEY_IP = "IP";
constexpr const char* KEY_ITEMCNT = "ItemCnt";
constexpr const char* KEY_USERNAME = "Username";
constexpr const char* KEY_PORT = "Port";
constexpr const char* HEAD_BASIC = "Basic";
constexpr const char* HEAD_ITEM = "Item";
constexpr const char* KEY_SOURCE = "Source";
constexpr const char* KEY_DES = "Des";
constexpr const char* KEY_ENABLE = "Enable";

ConfigOpr::ConfigOpr()
{
    std::string home = get_home();
    fs::path    config_dir = fs::path(home).append(".config").append("FTPTrans");
    auto        ini_path = fs::path(config_dir).append("config.ini");
    std::snprintf(ini_path_, sizeof(ini_path_), "%s", ini_path.string().c_str());

    if (!fs::exists(config_dir)) {
        fs::create_directories(config_dir);
    }
    if (!fs::exists(ini_path_)) {
        generate_default();
        return;
    }
}

void ConfigOpr::push(const ItemContent& item)
{
    check_open();

    std::string new_name(HEAD_ITEM);
    new_name += std::to_string(item.id);

    ini_.SetValue(new_name.c_str(), KEY_SOURCE, item.source.c_str());
    ini_.SetValue(new_name.c_str(), KEY_DES, item.des.c_str());
    ini_.SetBoolValue(new_name.c_str(), KEY_ENABLE, true);

    save();
}

bool ConfigOpr::update(int id, const ItemContent* ptem, bool enable)
{
    check_open();

    long cnt = ini_.GetLongValue(HEAD_BASIC, KEY_ITEMCNT);
    if (id > cnt) {
        return false;
    }

    std::string new_name(HEAD_ITEM);
    new_name += std::to_string(id);

    if (!enable) {
        bool ret = ini_.SetBoolValue(new_name.c_str(), KEY_ENABLE, false) == SI_UPDATED;
        assert(ret);
        save();
        return true;
    }

    if (!ptem) {
        return false;
    }

    ini_.SetValue(new_name.c_str(), KEY_SOURCE, ptem->source.c_str());
    ini_.SetValue(new_name.c_str(), KEY_DES, ptem->des.c_str());
    ini_.SetBoolValue(new_name.c_str(), KEY_ENABLE, true);

    save();

    return true;
}

long ConfigOpr::get_count()
{
    check_open();

    long cnt = ini_.GetLongValue(HEAD_BASIC, KEY_ITEMCNT);
    return cnt;
}

bool ConfigOpr::get_content(int id, ItemContent& item)
{
    check_open();

    long cnt = ini_.GetLongValue(HEAD_BASIC, KEY_ITEMCNT);
    if (id > cnt) {
        return false;
    }
    std::string new_name(HEAD_ITEM);
    new_name += std::to_string(id);

    item.enable = ini_.GetBoolValue(new_name.c_str(), KEY_ENABLE);
    item.des = ini_.GetValue(new_name.c_str(), KEY_DES);
    item.source = ini_.GetValue(new_name.c_str(), KEY_SOURCE);
    item.id = id;
    return true;
}

bool ConfigOpr::get_base(ItemBase& base)
{
    check_open();
    base.cnt = ini_.GetLongValue(HEAD_BASIC, KEY_ITEMCNT);
    base.ip.append(ini_.GetValue(HEAD_BASIC, KEY_IP));
    base.port = std::stoi(ini_.GetValue(HEAD_BASIC, KEY_PORT));
    base.pwd.append(ini_.GetValue(HEAD_BASIC, KEY_PWD));
    base.user.append(ini_.GetValue(HEAD_BASIC, KEY_USERNAME));
    return true;
}

bool ConfigOpr::set_base(const ItemBase& base)
{
    check_open();
    ini_.SetLongValue(HEAD_BASIC, KEY_ITEMCNT, base.cnt);
    ini_.SetValue(HEAD_BASIC, KEY_IP, base.ip.c_str());
    ini_.SetLongValue(HEAD_BASIC, KEY_PORT, base.port);
    ini_.SetValue(HEAD_BASIC, KEY_USERNAME, base.user.c_str());
    ini_.SetValue(HEAD_BASIC, KEY_PWD, base.pwd.c_str());
    save();
    return true;
}

std::string ConfigOpr::get_home()
{
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
    return std::getenv("USERPROFILE");
#else
    const char* homedir = getenv("HOME");
    if (homedir) {
        return std::string(homedir);
    }
    return "";
#endif
}

void ConfigOpr::generate_default()
{
    ini_.SetLongValue(HEAD_BASIC, KEY_ITEMCNT, 0);
    ini_.SetValue(HEAD_BASIC, KEY_IP, "192.168.0.9");
    ini_.SetLongValue(HEAD_BASIC, KEY_PORT, 21);
    ini_.SetValue(HEAD_BASIC, KEY_USERNAME, "demouser");
    ini_.SetValue(HEAD_BASIC, KEY_PWD, "123456");
    ini_.SaveFile(ini_path_);
}

void ConfigOpr::check_open()
{
    if (ini_.IsEmpty()) {
        bool ret = (ini_.LoadFile(ini_path_) == SI_OK);
        assert(ret);
    }
}

void ConfigOpr::save()
{
    bool ret = ini_.SaveFile(ini_path_) == SI_OK;
    assert(ret);
}
