#include "FTPTrans.h"
#include <curl/curl.h>

FTPTrans::FTPTrans(QWidget* parent) : QWidget(parent)
{
    ui.setupUi(this);

    ini_opr_ = new ConfigOpr();

    setWindowTitle("FTPTrans v1.0.2");

    init();

    connect(ui.btnSave, &QPushButton::clicked, this, [&]() { save(true); });
    connect(ui.btnAdd, &QPushButton::clicked, [&]() { handle_table_add(); });
    connect(ui.btnUpdate, &QPushButton::clicked, [&]() { Trans(); });
    connect(ui.btnDown, &QPushButton::clicked, [&]() { Trans(false); });
}

void FTPTrans::save(bool is_notify)
{
    ini_base_.ip = ui.edIP->text().toStdString();
    ini_base_.port = ui.edPort->text().toInt();
    ini_base_.pwd = ui.edPwd->text().toStdString();
    ini_base_.user = ui.edUserName->text().toStdString();
    ini_base_.cnt = ui.tableWidget->rowCount();

    ini_opr_->set_base(ini_base_);

    for (int i = 0; i < ini_base_.cnt; ++i) {
        ItemContent content;
        content.id = ui.tableWidget->item(i, 0)->text().toInt();
        content.source.append(ui.tableWidget->item(i, 1)->text().toLocal8Bit());
        content.des.append(ui.tableWidget->item(i, 2)->text().toLocal8Bit());
        ini_opr_->update(content.id, &content);
    }

    if (is_notify) {
        QMessageBox::information(this, u8"提示", u8"已存储");
    }
}

FTPTrans::~FTPTrans()
{
    delete ini_opr_;
}

size_t readFunc(void* ptr, size_t size, size_t nmemb, FILE* stream)
{
    return fread(ptr, size, nmemb, stream);
}

size_t writeFunc(void* ptr, size_t size, size_t nmemb, FILE* stream)
{
    return fwrite(ptr, size, nmemb, stream);
}

void FTPTrans::Trans(bool is_up)
{
    if (is_up) {
        if (!is_ok(this, u8"确认上传同步吗？")) {
            return;
        }
    } else {
        if (!is_ok(this, u8"确认下载同步吗？")) {
            return;
        }
    }

    CURL* curl = nullptr;

    QList<QTableWidgetItem*> selectedItems = ui.tableWidget->selectedItems();
    if (selectedItems.size() != 3) {
        QMessageBox::information(this, u8"提示", u8"未有选中数据");
        return;
    }

    std::string source(selectedItems[1]->text().toLocal8Bit());
    std::string des(selectedItems[2]->text().toLocal8Bit());
    std::string user(ui.edUserName->text().toStdString());
    std::string pwd(ui.edPwd->text().toStdString());

    try {
        curl_global_init(CURL_GLOBAL_ALL);

        std::uintmax_t file_size{};
        if (is_up) {
            fs::path source_file(source);
            if (!fs::exists(source_file)) {
                QMessageBox::information(this, u8"提示", QString::fromStdString(u8"文件不存在：" + source_file.string()));
                return;
            }
            file_size = fs::file_size(source_file);
        }

        curl = curl_easy_init();
        if (curl == nullptr) {
            QMessageBox::information(this, u8"提示", u8"curl_easy_init 初始化失败");
            return;
        }

        FILE* fd = nullptr;
        if (is_up) {
            fd = fopen(source.c_str(), "rb");
        } else {
            fd = fopen(source.c_str(), "wb");
        }

        if (!fd) {
            QMessageBox::information(this, u8"提示", QString::fromStdString(u8"文件打开失败：" + source));
            return;
        }

        if (is_up) {
            curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
        }

        curl_easy_setopt(curl, CURLOPT_URL, des.c_str());
        curl_easy_setopt(curl, CURLOPT_USERNAME, user.c_str());
        curl_easy_setopt(curl, CURLOPT_PASSWORD, pwd.c_str());

        if (is_up) {
            curl_easy_setopt(curl, CURLOPT_READFUNCTION, readFunc);
            curl_easy_setopt(curl, CURLOPT_READDATA, fd);
        } else {
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writeFunc);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, fd);
        }

        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);

        if (is_up) {
            curl_easy_setopt(curl, CURLOPT_INFILESIZE_LARGE, file_size);
        }

        if (10) {
            curl_easy_setopt(curl, CURLOPT_FTP_RESPONSE_TIMEOUT, 10);
        }

        CURLcode r = curl_easy_perform(curl);
        fclose(fd);

        if (r != CURLE_OK) {
            QMessageBox::information(this, u8"错误提示", QString::fromLocal8Bit(curl_easy_strerror(r)));
            return;
        }
    } catch (...) {
        QMessageBox::information(this, u8"异常", u8"产生了异常");
        return;
    }

    if (curl) {
        curl_easy_cleanup(curl);
    }
    curl_global_cleanup();
    save(false);
    QMessageBox::information(this, u8"提示", u8"完成");
}

void FTPTrans::init()
{
    ini_opr_->get_base(ini_base_);
    ui.edIP->setText(QString::fromStdString(ini_base_.ip));
    ui.edPort->setText(QString::number(ini_base_.port));
    ui.edPwd->setText(QString::fromStdString(ini_base_.pwd));
    ui.edUserName->setText(QString::fromStdString(ini_base_.user));

    QStringList headers;
    headers << u8"标识号" << u8"源位置（本地）" << u8"目的位置（FTP位置）";
    ui.tableWidget->setColumnCount(headers.size());
    ui.tableWidget->setHorizontalHeaderLabels(headers);

    ui.tableWidget->setColumnWidth(0, 80);
    ui.tableWidget->setColumnWidth(1, 450);
    ui.tableWidget->horizontalHeader()->setStretchLastSection(true);
    ui.tableWidget->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
    ui.tableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui.tableWidget->verticalHeader()->setVisible(false);

    ui.btnAdd->setMinimumHeight(35);
    ui.btnDown->setMinimumHeight(35);
    ui.btnSave->setMinimumHeight(35);
    ui.btnUpdate->setMinimumHeight(35);

    handle_table_read();
}

void FTPTrans::handle_table_read()
{
    int cnt = ini_opr_->get_count();
    for (int i = 1; i < cnt + 1; ++i) {
        ItemContent content;
        ini_opr_->get_content(i, content);
        int rowCnt = ui.tableWidget->rowCount();
        ui.tableWidget->insertRow(rowCnt);
        QTableWidgetItem* item0 = new QTableWidgetItem(QString::number(content.id));
        item0->setFlags(item0->flags() & ~Qt::ItemIsEditable);
        ui.tableWidget->setItem(rowCnt, 0, item0);
        QTableWidgetItem* item1 = new QTableWidgetItem(QString::fromLocal8Bit(content.source.c_str()));
        ui.tableWidget->setItem(rowCnt, 1, item1);
        QTableWidgetItem* item2 = new QTableWidgetItem(QString::fromLocal8Bit(content.des.c_str()));
        ui.tableWidget->setItem(rowCnt, 2, item2);
    }
}

void FTPTrans::handle_table_add()
{
    int rowCnt = ui.tableWidget->rowCount();
    ui.tableWidget->insertRow(rowCnt);
    QTableWidgetItem* item0 = new QTableWidgetItem(QString::number(rowCnt + 1));
    item0->setFlags(item0->flags() & ~Qt::ItemIsEditable);
    ui.tableWidget->setItem(rowCnt, 0, item0);
    QTableWidgetItem* item1 = new QTableWidgetItem("");
    ui.tableWidget->setItem(rowCnt, 1, item1);
    QTableWidgetItem* item2 = new QTableWidgetItem("");
    ui.tableWidget->setItem(rowCnt, 2, item2);
}

bool FTPTrans::is_ok(QWidget* parent, const QString& content)
{
    QMessageBox questionBox(parent);
    questionBox.setText(content);
    questionBox.setWindowTitle(u8"提示");
    questionBox.setIcon(QMessageBox::Question);
    questionBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    int result = questionBox.exec();
    if (result != QMessageBox::Yes) {
        return false;
    } else {
        return true;
    }
}
